using System.Drawing;
using System.Diagnostics;
using System.Windows.Forms;
using System.Collections;
using Microsoft.VisualBasic;
using System.Collections.Generic;
using System;


namespace EmployeeApp
{
	public class UIControl
	{
		
		
		//**
		//** Events
		//**
		public delegate void ShowSalaryUIEventHandler();
		private ShowSalaryUIEventHandler ShowSalaryUIEvent;
		
		public event ShowSalaryUIEventHandler ShowSalaryUI
		{
			add
			{
				ShowSalaryUIEvent = (ShowSalaryUIEventHandler) System.Delegate.Combine(ShowSalaryUIEvent, value);
			}
			remove
			{
				ShowSalaryUIEvent = (ShowSalaryUIEventHandler) System.Delegate.Remove(ShowSalaryUIEvent, value);
			}
		}
		
		public delegate void HideSalaryUIEventHandler();
		private HideSalaryUIEventHandler HideSalaryUIEvent;
		
		public event HideSalaryUIEventHandler HideSalaryUI
		{
			add
			{
				HideSalaryUIEvent = (HideSalaryUIEventHandler) System.Delegate.Combine(HideSalaryUIEvent, value);
			}
			remove
			{
				HideSalaryUIEvent = (HideSalaryUIEventHandler) System.Delegate.Remove(HideSalaryUIEvent, value);
			}
		}
		
		
		public delegate void SwitchingListControlEventHandler(IEmployeeList listControl, string menuText);
		private SwitchingListControlEventHandler SwitchingListControlEvent;
		
		public event SwitchingListControlEventHandler SwitchingListControl
		{
			add
			{
				SwitchingListControlEvent = (SwitchingListControlEventHandler) System.Delegate.Combine(SwitchingListControlEvent, value);
			}
			remove
			{
				SwitchingListControlEvent = (SwitchingListControlEventHandler) System.Delegate.Remove(SwitchingListControlEvent, value);
			}
		}
		
		
		public delegate void EmptyTextFieldEventHandler(TextBox txt);
		private EmptyTextFieldEventHandler EmptyTextFieldEvent;
		
		public event EmptyTextFieldEventHandler EmptyTextField
		{
			add
			{
				EmptyTextFieldEvent = (EmptyTextFieldEventHandler) System.Delegate.Combine(EmptyTextFieldEvent, value);
			}
			remove
			{
				EmptyTextFieldEvent = (EmptyTextFieldEventHandler) System.Delegate.Remove(EmptyTextFieldEvent, value);
			}
		}
		
		public delegate void ValidTextFieldEventHandler(TextBox txt);
		private ValidTextFieldEventHandler ValidTextFieldEvent;
		
		public event ValidTextFieldEventHandler ValidTextField
		{
			add
			{
				ValidTextFieldEvent = (ValidTextFieldEventHandler) System.Delegate.Combine(ValidTextFieldEvent, value);
			}
			remove
			{
				ValidTextFieldEvent = (ValidTextFieldEventHandler) System.Delegate.Remove(ValidTextFieldEvent, value);
			}
		}
		
		public delegate void EmployeeNotFoundEventHandler(string firstname, string lastname);
		private EmployeeNotFoundEventHandler EmployeeNotFoundEvent;
		
		public event EmployeeNotFoundEventHandler EmployeeNotFound
		{
			add
			{
				EmployeeNotFoundEvent = (EmployeeNotFoundEventHandler) System.Delegate.Combine(EmployeeNotFoundEvent, value);
			}
			remove
			{
				EmployeeNotFoundEvent = (EmployeeNotFoundEventHandler) System.Delegate.Remove(EmployeeNotFoundEvent, value);
			}
		}
		
		
		public delegate void EmployeeOpenedEventHandler(string firstname, string lastname);
		private EmployeeOpenedEventHandler EmployeeOpenedEvent;
		
		public event EmployeeOpenedEventHandler EmployeeOpened
		{
			add
			{
				EmployeeOpenedEvent = (EmployeeOpenedEventHandler) System.Delegate.Combine(EmployeeOpenedEvent, value);
			}
			remove
			{
				EmployeeOpenedEvent = (EmployeeOpenedEventHandler) System.Delegate.Remove(EmployeeOpenedEvent, value);
			}
		}
		
		public delegate void EmployeeClosedEventHandler(string firstname, string lastname);
		private EmployeeClosedEventHandler EmployeeClosedEvent;
		
		public event EmployeeClosedEventHandler EmployeeClosed
		{
			add
			{
				EmployeeClosedEvent = (EmployeeClosedEventHandler) System.Delegate.Combine(EmployeeClosedEvent, value);
			}
			remove
			{
				EmployeeClosedEvent = (EmployeeClosedEventHandler) System.Delegate.Remove(EmployeeClosedEvent, value);
			}
		}
		
		
		public delegate void ExitTheAppEventHandler();
		private ExitTheAppEventHandler ExitTheAppEvent;
		
		public event ExitTheAppEventHandler ExitTheApp
		{
			add
			{
				ExitTheAppEvent = (ExitTheAppEventHandler) System.Delegate.Combine(ExitTheAppEvent, value);
			}
			remove
			{
				ExitTheAppEvent = (ExitTheAppEventHandler) System.Delegate.Remove(ExitTheAppEvent, value);
			}
		}
		
		
		//**
		//** Fields
		//**
		private Employees employees; //** Model
		private FormMain frmMain; //** View: main form
		private ArrayList frmEmployees; //** View: sub-forms for employees
		private IEmployeeList curListControl; //** View: UI control user prefers on the main form
		
		//**
		//** Constructor: initialize presentation and business tiers to work with controller
		//**
		public UIControl(FormMain frmMain)
		{
			//**
			//** create/initialize objects related to various tiers of application...
			//**
			this.employees = new Employees(); //** model
			this.frmMain = frmMain; //** view
			this.frmEmployees = new ArrayList();
			this.curListControl = null;
		}
		
		//**
		//** Start the controller processing of user input
		//**
		public void Start()
		{
			//**
			//** hookup observers to receive user input events from main form...
			//**
			//** validation...
			//**
			this.frmMain.txtFirstName.Validating += new System.ComponentModel.CancelEventHandler(this.NonEmpty_Validating);
			this.frmMain.txtLastName.Validating += new System.ComponentModel.CancelEventHandler(this.NonEmpty_Validating);
			//**
			//** lookup button
			//**
			this.frmMain.cmdLookup.Click += new System.EventHandler(this.Lookup);
			//**
			//** menu events...
			//**
			this.frmMain.ExitToolStripMenuItem.Click += new System.EventHandler(this.ExitApp);
			this.frmMain.UseListBoxToolStripMenuItem.Click += new System.EventHandler(this.UseListBoxForEmployees);
			
			//**
			//** setup presentation based on user's preferences
			//**
			//** view salary info:
			//**
			if (Settings.Default.ViewSalaryInfo)
			{
				if (ShowSalaryUIEvent != null)
					ShowSalaryUIEvent();
				this.frmMain.ViewSalaryToolStripMenuItem.Click += new System.EventHandler(this.HideSalary);
			}
			else
			{
				if (HideSalaryUIEvent != null)
					HideSalaryUIEvent();
				this.frmMain.ViewSalaryToolStripMenuItem.Click += new System.EventHandler(this.ShowSalary);
			}
			
			//**
			//** control for viewing list of open employees:
			//**
			System.Reflection.Assembly asm;
			object obj;
			asm = System.Reflection.Assembly.Load(Settings.@Default.EmployeesListControlAssembly);
			obj = asm.CreateInstance(Settings.@Default.EmployeesListControl);
			
			this.curListControl = obj as IEmployeeList;
            if (curListControl == null)
            {
                throw (new ApplicationException("Can't case curListControl to IEmployeeList"));
            }
			
			//** hook in new control so we know when user clicks on employee in list
			this.curListControl.ClickOnEmployee += new IEmployeeListClickHandler(this.IEmployeeListClickHandler);
			
			if (Settings.@Default.EmployeesListControl.EndsWith("MyListBox"))
			{
				if (SwitchingListControlEvent != null)
					SwitchingListControlEvent(this.curListControl, "Use Table");
				this.frmMain.UseListBoxToolStripMenuItem.Click += new System.EventHandler(this.UseTableLayoutPanelForEmployees);
			}
			else if (Settings.@Default.EmployeesListControl.EndsWith("MyTableLayoutPanel"))
			{
				if (SwitchingListControlEvent != null)
					SwitchingListControlEvent(this.curListControl, "Use ListBox");
				this.frmMain.UseListBoxToolStripMenuItem.Click += new System.EventHandler(this.UseListBoxForEmployees);
			}
			else
			{
				throw (new ApplicationException("Unknown UI control for listing employees on main form?!"));
			}
			
			//**
			//** now recreate list of open employees when user exited:
			//**
			if (Settings.@Default.EmployeesOpen == null) //** first time, there is no list of open employees
			{
				Settings.@Default.EmployeesOpen = new ArrayList();
			}
			else
			{
				string fn;
				string ln;
				string fullname;
				Employee emp;
				string[] names;
				
				foreach (string tempLoopVar_fullname in Settings.@Default.EmployeesOpen)
				{
					fullname = tempLoopVar_fullname;
					names = fullname.Split(null);
					ln = names[0].Substring(0, names[0].Length - 1);
					fn = names[1];
					
					emp = this.employees.Lookup(fn, ln);
					
					if (emp == null)
					{
						//** not found, ignore...
					}
					else
					{
						if (EmployeeOpenedEvent != null)
							EmployeeOpenedEvent(emp.FirstName, emp.LastName);
						
						FormEmployeeInfo frm;
						frm = new FormEmployeeInfo(emp, this, Settings.@Default.ViewSalaryInfo);
						
						//** keep track of open sub-forms...
						this.frmEmployees.Add(frm);
						
						//** add handler so we know when it's closed...
						frm.FormClosed += new System.Windows.Forms.FormClosedEventHandler(this.FormEmployeeInfo_Closed);
						
						frm.Show();
					}
				}
			}
		}
		
		//**
		//** Validation of a non-empty text field
		//**
		public void NonEmpty_Validating(System.Object sender, System.ComponentModel.CancelEventArgs e)
		{
			TextBox txt;
			txt = (TextBox) sender;
			
			if (txt.Text.Trim() == "")
			{
				if (EmptyTextFieldEvent != null)
					EmptyTextFieldEvent(txt);
			}
			else
			{
				if (ValidTextFieldEvent != null)
					ValidTextFieldEvent(txt);
			}
		}
		
		//**
		//** Exit the application
		//**
		public void ExitApp(System.Object sender, System.EventArgs e)
		{
			//** save user's settings!
			Settings.@Default.Save();
			
			//** now shutdown the presentation tier...
			if (ExitTheAppEvent != null)
				ExitTheAppEvent();
		}
		
		//**
		//** Called when user wants to lookup an employee
		//**
		private void Lookup(System.Object sender, System.EventArgs e)
		{
			string fn;
			string ln;
			Employee emp;
			
			fn = this.frmMain.FirstName;
			ln = this.frmMain.LastName;
			
			emp = this.employees.Lookup(fn, ln);
			
			if (emp == null)
			{
				if (EmployeeNotFoundEvent != null)
					EmployeeNotFoundEvent(fn, ln);
			}
			else if (this.isEmployeeFormAlreadyOpen(emp.FirstName, emp.LastName))
			{
				//** nothing, frm has been activated
			}
			else
			{
				//** not yet open, so go ahead and open a new sub-form for this employee...
				if (EmployeeOpenedEvent != null)
					EmployeeOpenedEvent(emp.FirstName, emp.LastName);
				
				FormEmployeeInfo frm;
				frm = new FormEmployeeInfo(emp, this, Settings.@Default.ViewSalaryInfo);
				
				//** keep track of open sub-forms...
				this.frmEmployees.Add(frm);
				Settings.@Default.EmployeesOpen.Add(emp.LastName + ", " + emp.FirstName);
				
				//** add handler so we know when it's closed...
				frm.FormClosed += new System.Windows.Forms.FormClosedEventHandler(this.FormEmployeeInfo_Closed);
				
				frm.Show();
			}
		}
		
		private bool isEmployeeFormAlreadyOpen(string firstname, string lastname)
		{
			FormEmployeeInfo frm;
			
			foreach (FormEmployeeInfo tempLoopVar_frm in this.frmEmployees)
			{
				frm = tempLoopVar_frm;
				if (firstname.Equals(frm.FirstName) && lastname.Equals(frm.LastName))
				{
					frm.Activate();
					return true;
				}
			}
			
			//** if get here, form is not open for this employee
			return false;
		}
		
		//**
		//** Called to dynamically show employee salaries
		//**
		private void ShowSalary(System.Object sender, System.EventArgs e)
		{
			Settings.@Default.ViewSalaryInfo = true; //** remember that user is viewing salary info
			
			if (ShowSalaryUIEvent != null)
				ShowSalaryUIEvent();
			
			//** toggle handlers...
			this.frmMain.ViewSalaryToolStripMenuItem.Click -= new System.EventHandler(this.ShowSalary);
			this.frmMain.ViewSalaryToolStripMenuItem.Click += new System.EventHandler(this.HideSalary);
		}
		
		//**
		//** Called to dynamically hide employee salaries
		//**
		private void HideSalary(System.Object sender, System.EventArgs e)
		{
			Settings.@Default.ViewSalaryInfo = false; //** remember that user is NOT viewing salary info
			
			if (HideSalaryUIEvent != null)
				HideSalaryUIEvent();
			
			//** toggle handlers...
			this.frmMain.ViewSalaryToolStripMenuItem.Click -= new System.EventHandler(this.HideSalary);
			this.frmMain.ViewSalaryToolStripMenuItem.Click += new System.EventHandler(this.ShowSalary);
		}
		
		//**
		//** Called to dynamically use a list box for displaying employees
		//**
		private void UseListBoxForEmployees(System.Object sender, System.EventArgs e)
		{
			Settings.@Default.EmployeesListControl = "EmployeeApp.MyListBox"; //** remember control user prefers
			Settings.@Default.EmployeesListControlAssembly = "EmployeeApp";
			
			this.SetupNewListControl("Use Table");
			
			//** toggle handlers for main form's menu item...
			this.frmMain.UseListBoxToolStripMenuItem.Click -= new System.EventHandler(this.UseListBoxForEmployees);
			this.frmMain.UseListBoxToolStripMenuItem.Click += new System.EventHandler(this.UseTableLayoutPanelForEmployees);
		}
		
		private void SetupNewListControl(string newMenuText)
		{
			this.curListControl.ClickOnEmployee -= new IEmployeeListClickHandler(this.IEmployeeListClickHandler);
			
			System.Reflection.Assembly asm;
			object obj;
			asm = System.Reflection.Assembly.Load(Settings.@Default.EmployeesListControlAssembly);
			obj = asm.CreateInstance(Settings.@Default.EmployeesListControl);
			
			this.curListControl = (IEmployeeList) obj;
			
			//** hook in new control so we know when user clicks on employee in list
			this.curListControl.ClickOnEmployee += new IEmployeeListClickHandler(this.IEmployeeListClickHandler);
			
			if (SwitchingListControlEvent != null)
				SwitchingListControlEvent(this.curListControl, newMenuText);
			
			//** reload new list control with open employees:
			FormEmployeeInfo frm;
			foreach (FormEmployeeInfo tempLoopVar_frm in this.frmEmployees)
			{
				frm = tempLoopVar_frm;
				if (EmployeeOpenedEvent != null)
					EmployeeOpenedEvent(frm.FirstName, frm.LastName);
			}
		}
		
		//**
		//** Called to dynamically use a table layout panel for displaying employees
		//**
		private void UseTableLayoutPanelForEmployees(System.Object sender, System.EventArgs e)
		{
			Settings.@Default.EmployeesListControl = "EmployeeApp.MyTableLayoutPanel"; //** remember control user prefers
			Settings.@Default.EmployeesListControlAssembly = "EmployeeApp";
			
			this.SetupNewListControl("Use ListBox");
			
			//** toggle handlers for main form's menu item...
			this.frmMain.UseListBoxToolStripMenuItem.Click -= new System.EventHandler(this.UseTableLayoutPanelForEmployees);
			this.frmMain.UseListBoxToolStripMenuItem.Click += new System.EventHandler(this.UseListBoxForEmployees);
		}
		
		//**
		//** Called when one of the employees on the main form is clicked upon:
		//**
		private void IEmployeeListClickHandler(string name)
		{
			//**
			//** find the open sub-form displaying this employee's info...
			//**
			FormEmployeeInfo frm;
			
			foreach (FormEmployeeInfo tempLoopVar_frm in this.frmEmployees)
			{
				frm = tempLoopVar_frm;
				if (name.Equals(frm.LastName + ", " + frm.FirstName))
				{
					frm.Activate();
                    break;
				}
			}
		}
		
		//**
		//** Called when one of the employee info sub-forms is closed
		//**
		private void FormEmployeeInfo_Closed(System.Object sender, System.Windows.Forms.FormClosedEventArgs e)
		{
			FormEmployeeInfo frm;
			frm = (FormEmployeeInfo) sender;
			
			if (EmployeeClosedEvent != null)
				EmployeeClosedEvent(frm.FirstName, frm.LastName);
			
			this.frmEmployees.Remove(frm); //** remove from our internal lists
			Settings.@Default.EmployeesOpen.Remove(frm.LastName + ", " + frm.FirstName);
		}
		
	}
	
}
