using System.Drawing;
using System.Diagnostics;
using System.Windows.Forms;
using System.Collections;
using Microsoft.VisualBasic;
using System.Collections.Generic;
using System;


namespace EmployeeApp
{
	public partial class FormEmployeeInfo
	{
		
		private UIControl controller;
		private Employee employee;
		
		//**
		//** Properties for retrieving firstname/lastname of employee being displayed
		//**
		public string FirstName
		{
			get
			{
				return this.employee.FirstName;
			}
		}
		
		public string LastName
		{
			get
			{
				return this.employee.LastName;
			}
		}
		
		//**
		//** Constructor
		//**
		public FormEmployeeInfo(Employee e, UIControl controller, bool showSalary)
		{
			
			this.InitializeComponent();
			
			this.employee = e;
			this.controller = controller;
			
			//**
			//** setup presentation of this employee's info:
			//**
			this.Text = this.employee.LastName + ", " + this.employee.FirstName;
			
			this.UpdateContactInfoTabPage();
			this.UpdateHomeAddrTabPage();
			
			if (showSalary)
			{
				this.ShowSalaryUI();
			}
			
			//**
			//** hook in to controller's events of interest:
			//**
			controller.ShowSalaryUI += new EmployeeApp.UIControl.ShowSalaryUIEventHandler(this.ShowSalaryUI);
			controller.HideSalaryUI += new EmployeeApp.UIControl.HideSalaryUIEventHandler(this.HideSalaryUI);
		}
		
		//**
		//** Triggered to dynamically show the "Salary" tab page.
		//**
		private void ShowSalaryUI()
		{
			//**
			//** Show salary tab, which we create on the fly
			//**
			TabPage tabpg;
			tabpg = new TabPage("Salary");
			tabpg.Name = "tabpgSalary";
			tabpg.ForeColor = Color.Red;
			
			Label lbl;
			lbl = new Label();
			lbl.Text = "Salary:";
			lbl.Location = new Point(30, 40);
			
			TextBox txt;
			txt = new TextBox();
			txt.Name = "txtSalary";
			txt.Location = new Point(100, 37);
			txt.ReadOnly = true;
			txt.Size = new Size(200, 26);
			
			tabpg.Controls.Add(txt);
			tabpg.Controls.Add(lbl);
			this.tabctrlEmployeeInfo.TabPages.Add(tabpg);
			
			this.UpdateSalaryTabPage();
		}
		
		//**
		//** Triggered to dynamically hide the "Salary" tab page created above
		//**
		private void HideSalaryUI()
		{
			//**
			//** Hide Salary tab by discarding what we created earlier...
			//**
			TabPage tabpg;
			tabpg = this.tabctrlEmployeeInfo.TabPages["tabpgSalary"];
			this.tabctrlEmployeeInfo.TabPages.Remove(tabpg);
		}
		
		//**
		//** These methods are called to display data about the employee
		//**
		private void UpdateContactInfoTabPage()
		{
			this.txtEmail.DataBindings.Clear();
			this.txtCellPhone.DataBindings.Clear();
			
			this.txtEmail.DataBindings.Add("Text", this.employee, "Email");
			this.txtCellPhone.DataBindings.Add("Text", this.employee, "CellPhone");
		}
		
		private void UpdateHomeAddrTabPage()
		{
			this.txtCity.DataBindings.Clear();
			
			this.txtCity.DataBindings.Add("Text", this.employee, "City");
		}
		
		private void UpdateSalaryTabPage()
		{
			TabPage tabpg;
			tabpg = this.tabctrlEmployeeInfo.TabPages["tabpgSalary"];
			
			tabpg.Controls["txtSalary"].DataBindings.Clear();
			tabpg.Controls["txtSalary"].DataBindings.Add("Text", this.employee, "FormattedSalary");
		}
		
	}
	
}
