using System.Drawing;
using System.Diagnostics;
using System.Windows.Forms;
using System.Collections;
using Microsoft.VisualBasic;
using System.Collections.Generic;
using System;


namespace EmployeeApp
{
	public partial class FormMain
	{
		public FormMain()
		{
			InitializeComponent();
		}
		
		private UIControl controller;
		private IEmployeeList listControl;
		
		//**
		//** Properties for retrieving firstname/lastname entered by user:
		//**
		public string FirstName
		{
			get
			{
				return this.txtFirstName.Text;
			}
		}
		
		public string LastName
		{
			get
			{
				return this.txtLastName.Text;
			}
		}
		
		//**
		//** Start of application and the presentation, in which case we immediately create and hand
		//** off to the controller...
		//**
		private void FormMain_Load(System.Object sender, System.EventArgs e)
		{
			this.controller = new UIControl(this);
			this.listControl = null;
			
			//**
			//** hookup handlers for events raised by controller
			//**
			controller.ExitTheApp += new EmployeeApp.UIControl.ExitTheAppEventHandler(this.FormMain_Exit);
			
			controller.EmptyTextField += new EmployeeApp.UIControl.EmptyTextFieldEventHandler(this.FormMain_EmptyTextField);
			controller.ValidTextField += new EmployeeApp.UIControl.ValidTextFieldEventHandler(this.FormMain_ValidTextField);
			
			controller.EmployeeNotFound += new EmployeeApp.UIControl.EmployeeNotFoundEventHandler(this.FormMain_EmployeeNotFound);
			controller.EmployeeOpened += new EmployeeApp.UIControl.EmployeeOpenedEventHandler(this.FormMain_EmployeeOpened);
			controller.EmployeeClosed += new EmployeeApp.UIControl.EmployeeClosedEventHandler(this.FormMain_EmployeeClosed);
			
			controller.ShowSalaryUI += new EmployeeApp.UIControl.ShowSalaryUIEventHandler(this.FormMain_ShowSalaryUI);
			controller.HideSalaryUI += new EmployeeApp.UIControl.HideSalaryUIEventHandler(this.FormMain_HideSalaryUI);
			
			controller.SwitchingListControl += new EmployeeApp.UIControl.SwitchingListControlEventHandler(this.FormMain_SwitchingListControl);
			
			//**
			//** okay, turn control over to the controller!
			//**
			this.controller.Start();
		}
		
		//**
		//** Called when user/control wants to shutdown the app...
		//**
		private void FormMain_Exit()
		{
			this.Close();
		}
		
		//**
		//** Called when the user fails to enter a valid name into text box
		//**
		private void FormMain_EmptyTextField(TextBox txt)
		{
			this.ErrorProviderTextBoxes.SetError(txt, "Please enter a value...");
		}
		
		//**
		//** Called when the user enters a valid name into text box
		//**
		private void FormMain_ValidTextField(TextBox txt)
		{
			this.ErrorProviderTextBoxes.SetError(txt, "");
		}
		
		//**
		//** Called when employee is not found
		//**
		private void FormMain_EmployeeNotFound(string firstname, string lastname)
		{
			MessageBox.Show("Employee not found: " + lastname + ", " + firstname, "Employee Application", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
			
			this.txtFirstName.SelectAll();
			this.txtFirstName.Focus();
		}
		
		//**
		//** Called when employee is found and a sub-form is opened to display his/her info
		//**
		private void FormMain_EmployeeOpened(string firstname, string lastname)
		{
			this.listControl.Add(lastname + ", " + firstname);
		}
		
		//**
		//** Called when sub-form displaying employee's info is closed
		//**
		private void FormMain_EmployeeClosed(string firstname, string lastname)
		{
			this.listControl.Remove(lastname + ", " + firstname);
		}
		
		//**
		//** Called to update GUI when we are showing salary info
		//**
		private void FormMain_ShowSalaryUI()
		{
			this.ViewSalaryToolStripMenuItem.Text = "Hide Salary"; //** flip menu text
		}
		
		//**
		//** Called to update GUI when we are NOT showing salary info
		//**
		private void FormMain_HideSalaryUI()
		{
			this.ViewSalaryToolStripMenuItem.Text = "View Salary"; //** flip menu text
		}
		
		//**
		//** Called when user wants to switch list controls...
		//**
		public void FormMain_SwitchingListControl(IEmployeeList listControl, string menuText)
		{
			this.UseListBoxToolStripMenuItem.Text = menuText;
			
			//**
			//** is this app startup time?  or are we switching controls at run-time?
			//**
			if (this.listControl != null) //** we are switching controls...
			{
				this.Controls.Remove((Control) this.listControl);
			}
			
			//**
			//** now setup to use the new control
			//**
			Control c;
			
			this.listControl = listControl;
			
			c = (Control) listControl;
			c.Font = this.cmdLookup.Font;
			c.Location = new System.Drawing.Point(this.cmdLookup.Location.X, this.cmdLookup.Location.Y + this.cmdLookup.Size.Height + 20);
			c.Size = new System.Drawing.Size(this.cmdLookup.Size.Width, this.Size.Height - c.Location.Y - 60);
			//c.Anchor = AnchorStyles.Bottom || AnchorStyles.Left || AnchorStyles.Right || AnchorStyles.Top;
			c.CausesValidation = false;
			
			this.Controls.Add(c);
		}

   
		
	}
}
